Index: res/res_musiconhold.c
===================================================================
RCS file: /usr/cvsroot/asterisk/res/res_musiconhold.c,v
retrieving revision 1.40
diff -u -r1.40 res_musiconhold.c
--- res/res_musiconhold.c	17 Nov 2004 18:16:08 -0000	1.40
+++ res/res_musiconhold.c	9 Dec 2004 01:20:46 -0000
@@ -588,7 +588,7 @@
 	return 0;
 }
 
-int ast_moh_start(struct ast_channel *chan, char *class)
+static int local_ast_moh_start(struct ast_channel *chan, char *class)
 {
 	if (!class || ast_strlen_zero(class))
 		class = chan->musicclass;
@@ -597,7 +597,7 @@
 	return ast_activate_generator(chan, &mohgen, class);
 }
 
-void ast_moh_stop(struct ast_channel *chan)
+static void local_ast_moh_stop(struct ast_channel *chan)
 {
 	ast_deactivate_generator(chan);
 }
@@ -659,6 +659,7 @@
 {
 	int res;
 	load_moh_classes();
+	ast_install_music_functions(local_ast_moh_start, local_ast_moh_stop);
 	res = ast_register_application(app0, moh0_exec, synopsis0, descrip0);
 	ast_register_atexit(ast_moh_destroy);
 	if (!res)
Index: channel.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channel.c,v
retrieving revision 1.149
diff -u -r1.149 channel.c
--- channel.c	7 Dec 2004 20:38:43 -0000	1.149
+++ channel.c	9 Dec 2004 01:20:48 -0000
@@ -24,6 +24,7 @@
 #include <asterisk/sched.h>
 #include <asterisk/options.h>
 #include <asterisk/channel.h>
+#include <asterisk/musiconhold.h>
 #include <asterisk/channel_pvt.h>
 #include <asterisk/logger.h>
 #include <asterisk/say.h>
@@ -2958,4 +2959,37 @@
 		}
 	}
 	return group;
+}
+
+
+static int (*ast_moh_start_ptr)(struct ast_channel *, char *) = NULL;
+static void (*ast_moh_stop_ptr)(struct ast_channel *) = NULL;
+
+
+void ast_install_music_functions(int (*start_ptr)(struct ast_channel *, char *),
+								void (*stop_ptr)(struct ast_channel *)) {
+	ast_moh_start_ptr = start_ptr;
+	ast_moh_stop_ptr = stop_ptr;
+}
+
+void ast_uninstall_music_functions(void) {
+	ast_moh_start_ptr = NULL;
+	ast_moh_stop_ptr = NULL;
+}
+
+/*! Turn on/off music on hold on a given channel */
+
+int ast_moh_start(struct ast_channel *chan, char *mclass) {
+	if(ast_moh_start_ptr)
+		return ast_moh_start_ptr(chan, mclass);
+
+	if (option_verbose > 2)
+		ast_log(LOG_WARNING,"Music class %s requested but no musiconhold loaded.\n", mclass ? mclass : "default");
+	
+	return 0;
+}
+
+void ast_moh_stop(struct ast_channel *chan) {
+	if(ast_moh_stop_ptr)
+		ast_moh_stop_ptr(chan);
 }
Index: include/asterisk/musiconhold.h
===================================================================
RCS file: /usr/cvsroot/asterisk/include/asterisk/musiconhold.h,v
retrieving revision 1.2
diff -u -r1.2 musiconhold.h
--- include/asterisk/musiconhold.h	29 Aug 2004 06:09:29 -0000	1.2
+++ include/asterisk/musiconhold.h	9 Dec 2004 01:20:51 -0000
@@ -24,6 +24,10 @@
 /*! Turn off music on hold on a given channel */
 extern void ast_moh_stop(struct ast_channel *chan);
 
+extern void ast_install_music_functions(int (*start_ptr)(struct ast_channel *, char *),
+								 void (*stop_ptr)(struct ast_channel *));
+extern void ast_uninstall_music_functions(void);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
